#!/usr/bin/perl
#
# --
# check_aruba_time - Check Time of an Aruba Switch (HP Switch)
#
# Copyright (C) 2016 Florian Fuchs (florian.fuchs@k.roteskreuz.at)
#
# --
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
# MA 02111-1307, USA.
#
#
# requires Net::SSH::Expect, HTTP::Date
#
# The script connects via SSH to an Aruba Switch (HP Switch), executes the "show time" command and then compares the result with 
# the current time. Depending on the chosen parameters it returns OK, WARNING or CRITICAL. In case of an error it returns 
# UNKNOWN.
#
# Since the Aruba Switch does not support the execution of commands via SSH, the program has to open a shell and write commands 
# and read their results. 
#

use strict;
use warnings;
use Net::SSH::Expect;
use HTTP::Date;

use Getopt::Long qw(:config no_ignore_case);	# we want the options to be case-sensitive

use constant VERSION => "1.1 (2016-06-08)";	# version number of the script
use constant LOGINTIMEOUT => 10;		# login timeout in seconds (it takes the Aruba Switch a while to login a user)
use constant PROMPTTIMEOUT => 2;		# timeout for the prompt to appear
use constant SHOWTIME => "show time\n";		# Aruba show time command

my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3);			# Nagios return values


my $host;		# host name or ip address of the Aruba Switch
my $user;		# SSH user name
my $password;		# SSH password
my $critical;		# critical deviation in seconds
my $warning;		# warning deviation in seconds
my $returnmessage;	# return message for Nagios

#
# usage(optional errormessage)
#
sub usage 
{
	my $message = $_[0];

	# if we have a message, add it to the output
	if (defined $message && length $message) 
	{
		# if there is no \n at the end of the string, add one
		$message .= "\n"
         	   unless $message =~ /\n$/;
   	}

	# get the script name
   	my $command = $0;
   	$command =~ s#^.*/##;

	#get version number
	my $version = VERSION;
	# print error message
   	print STDERR <<EOT;

$command (Version $version)

$message
usage: $command -H <host_address> -u <username> -p <password> [-w variance in seconds] [-c variance in seconds]

     Examples:
        check_aruba_time.pl -H 172.27.1.2 -u user -p pass -c 5
        check_aruba_time.pl -H host.domain.at -u user -p pass -w 5
        check_aruba_time.pl -H host.domain.at -u user -p pass -w 5 -c 10

EOT
	# exit the script
   	NagExit("UNKNOWN");
}


#
# usage (OK|WARNING|CRITICAL|UNKNOWN, [message]
#
sub NagExit 
{
	my $exitcode = $_[0];
	my $message = $_[1];

        # if we have a message, add it to the output
        if (defined $message && length $message)
        {
                # if there is no \n at the end of the string, add one
                $message .= "\n"
                   unless $message =~ /\n$/;

		if ($exitcode eq "OK")
		{
			$message = "OK: ".$message;
		}
		elsif ($exitcode eq "WARNING")
		{
			$message = "WARNING: ".$message;
		}
		elsif ($exitcode eq "CRITICAL")
		{
			$message = "CRITICAL: ".$message;
		}
		elsif ($exitcode eq "UNKNOWN")
		{
			$message = "UNKNOWN: ".$message;
		}
		print "$message";
      }

	exit $ERRORS{$exitcode};
}

# get the arguments
GetOptions ("H=s" => \$host,    			# mandatory string
            "u=s" => \$user,      			# mandatory string
            "p=s" => \$password,			# mandatory string
	    "c=i" => \$critical,			# mandatory integer (if option is chosen)
            "w=i" => \$warning) 			# mandatory integer (if option is chosen)

or usage("Invalid commmand line options.");
 
# check if we have all mandatory arguments
usage("The host must be specified.")
   unless defined $host;

usage("The user must be specified.")
   unless defined $user;

usage("The password must be specified.")
   unless defined $password;

my $ssh = Net::SSH::Expect->new (
	host => "$host", 
	password=> "$password", 
	user => "$user", 
	raw_pty => 1,
	timeout => LOGINTIMEOUT,
	terminator => "\r"
        );

# check if host exists
NagExit("UNKNOWN", "Host $host not found.") unless gethostbyname($host);


my $login_output;		# output from login

# try to login
eval
{
	$login_output = $ssh->login();
}
# if there is an error, exit
or do
{
	NagExit("UNKNOWN", "Could not login via SSH: $@");
};

# the Aruba Switch will ask the user to "Presse any key to continue" if the login is successful
if ($login_output !~ /Press any key to continue/)
{
	NagExit("UNKNOWN", "Incorrect login or no Aruba Swirch!");
}

# press any key to continue (we send "a")
$ssh->send("a");

# wait for the prompt ("# " or "> " depending on the type of user)
unless($ssh->waitfor('(#\s|>\s)', PROMPTTIMEOUT))
{
	$ssh->close();
	NagExit("UNKNOWN", "Shell prompt not found.");
};

# read anything we have received so far (we don't need it)
$ssh->read_line(5);

# send "show time"
$ssh->send(SHOWTIME);

# read everything
my $serveranswer = $ssh->read_line(5);

# close the connection
$ssh->close();

# The Aruba Switch returns lots of things we are not interested in. Additionally, it uses the escape
# character as a line break. 
my @lines = split /\e/, $serveranswer;

# We are only interested in the last line
my $timestring = $lines[$#lines];

# get rid of rubbish at the head of the string
$timestring =~ s/\[24\;1H//g;

# convert timestring to datetime
my $arubatime = str2time($timestring);

# get current time and convert it to datetime 
my $now = str2time(localtime);

# calculate the deviation
my $deviation = $arubatime-$now;

# if there is no deviation
if ($deviation == 0)
{
	$returnmessage = "The Aruba Switch has the same time as the Nagios server. The switch reported $timestring as current time.\n";
}
# if the deviation is positive the Aruba Switch is early
elsif($deviation == abs($deviation))
{
	$returnmessage = "The Aruba Switch is ".$deviation." seconds early. The switch reported $timestring as current time.\n";
}
# if the deviation is negative the Aruba Switch is late
else
{
	$returnmessage = "The Aruba Switch is ".abs($deviation)." seconds late. The switch reported $timestring as current time.\n";
}


# if we defined a critical level and we exceed it
if (defined $critical && abs($deviation) >= $critical)
{
	NagExit("CRITICAL", $returnmessage);
}
# if we defined a warning level and exceed it
elsif (defined $warning && abs($deviation) >= $warning)
{
	NagExit("WARNING", $returnmessage);
}
# in all other cases everything is fine
else
{
	NagExit("OK", $returnmessage);
}
